package edu.toronto.cs.experiments;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import org.apache.log4j.*;
import edu.toronto.cs.algebra.*;

/** A class representing sparse functions, where most of the inputs
 *  have false (0) as an output. */
public abstract class Function {

    static protected Category log = 
	Category.getInstance("edu.toronto.cs.experiments.Function");

    int m;
    int k;

    /** @param m: size of internal variables
     *  @param k: number of variables
     */
    public Function(int m_, int k_) {
	m=m_;
	k=k_;

    }

    protected abstract void fill();
    
    public abstract Function clone(int m_, int k_);

    public void refill() {
	fill();
    }

    public class Entry {
	public int inputs[];
	public int output;
	public Entry(Object inp[], int o) {
	    inputs = new int[inp.length];
	    for (int i=0; i<inp.length;i++)
		inputs[i] = ((Integer)inp[i]).intValue();
	    output = o;
	}

	public int hashCode() {
	    int hc = 0;
	    for (int i=0; i<inputs.length; i++)
		hc += new Integer(inputs[i]).hashCode();
	    return hc;
	}
	public boolean equals(Object o2) {
	    if (!(o2 instanceof Function))
		return false;
	    else {
		Entry e = (Entry)o2;
		boolean res = true;
	        int i =0;
		while (res && i<inputs.length)
		    {
			res = e.inputs[i] == inputs[i];
			i++;
		    }
		return res;
	    }
	}
	public String toString() {
	    String x;
	    if (inputs[0] == -1)
		x = new String("-");
	    else
		x = Integer.toString(inputs[0]);
	    for (int k=1; k<inputs.length; k++)
		if (inputs[k] == -1)
		    x = new String(x+",-");
		else
		    x = new String(x+","+inputs[k]);
	    return new String("f("+x+")="+output);
	}
      public String toString (FiniteLattice lattice)
      {
	StringBuffer sb = new StringBuffer ();
	for (int i = 0; i < inputs.length; i++)
	  if (inputs [i] == Algebra.NO_VALUE)
	    sb.append ('-');
	  else
	    sb.append (lattice.getElementKeyByIdx (inputs [i]));
	sb.append (' ');
	if (output == Algebra.NO_VALUE)
	  sb.append ('-');
	else
	  sb.append (lattice.getElementKeyByIdx (output));

	return sb.toString ();
      }
      
    }

    public static String intArrayToString(int q[]) {
	String x = Integer.toString(q[0]);
	for (int k=1; k<q.length; k++)
	    x = new String(x+","+q[k]);
	return x;
    }
    public static int[] objArrayToIntArray(Object o[]) {
	int x[] = new int[o.length];
	for (int k=0; k<o.length; k++)
	    x[k] = ((Integer)o[k]).intValue();
	return x;
    }


    // everything not in the map is assumed to be zero

    public abstract Iterator getAllPoints();

    public abstract Function embed(int newm, int mapping[]);


    protected int powr(int i, int j) {

	return (int)(Math.pow((double)i, (double)j));
    }
}

