package edu.toronto.cs.experiments;

import edu.toronto.cs.mvset.*;
import edu.toronto.cs.algebra.*;
import java.util.*;


/***
 *** MultiValued function
 ***/
public abstract class MvFunction
{
  FiniteLattice lattice;
  int arrity;

  int bot;
  
  
  public MvFunction (FiniteLattice _lattice, int _arrity)
  {
    lattice = _lattice;
    arrity = _arrity;

    bot = lattice.idplusIdx ();
  }
  
  public String mintermToString (int[] minterm)
  {
    StringBuffer sb = new StringBuffer ();
    for (int i = 0; i < minterm.length; i++)
      {
	sb.append (valueToString (minterm [i]));
	sb.append (' ');
      }
    

    return sb.toString ();
  }
  
  public String valueToString (int val)
  {
    return val == Algebra.NO_VALUE ? 
      "-" : lattice.getElementKeyByIdx (val).toString ();
    
  }
  
  
  
  
  /*** Convert this function into its MvSet representation ***/
  public abstract MvSet toMvSet (MvSetFactory factory);
  
  /*** Generate new MvFunction */
  public abstract MvFunction newMvFunction ();
  
  public class MvEntry
  {
    int[] minterm;
    int value;
    
    public MvEntry (int[] _minterm, int _value)
    {
      minterm = _minterm;
      value = _value;
    }
    
    public int[] getMinterm ()
    {
      return minterm;
    }
    public int getValue ()
    {
      return value;
    }
    
    public MvSet toMvSet (MvSetFactory factory)
    {
      return factory.createPoint (minterm, value);
    }
    
    public boolean equals (Object o)
    {
      if (o!= null && o instanceof MvEntry) return equals ((MvEntry)o);
      return false;
    }
    
    public boolean equals (MvEntry v)
    {
      if (value != v.value) return false;
      if (minterm.length != v.minterm.length) return false;
      
      for (int i = 0; i < minterm.length; i++)
	if (minterm [i] != v.minterm [i]) return false;
      return true;
    }
    
    public int hashCode ()
    {
      int hash = value;
      for (int i = 0; i < minterm.length; i++)
	hash += minterm [i];
      return hash;
    }
    
	
  }
  
}

