package edu.toronto.cs.gui;

import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.ctl.*;
import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.tlq.*;
import edu.toronto.cs.smv.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.keg.*;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.prettygraph.PrettyConstants;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;

import javax.swing.table.*;

/**
 * LatticeTables.java
 *
 * An extension of JPanel. Contains a tabbed panel with three tabs:
 * to display negation, meet, and join tables for a particular lattice.
 *
 * Created: Tue Jun 25 11:45:09 2002
 *
 * @author <a href="mailto:anya@tallinn.cs">Tafliovich Anya</a>
 * @version
 */

public class LatticeDisplay extends JFrame {

  // the tabbed panel
  JTabbedPane tabs;
  // the image panel  
  ImagePanel image;
  // the close button
  JButton close;

  public LatticeDisplay (FiniteLattice lattice)
  {
    super("Lattice");
    update (lattice);
  }

  // initialize a tabbed panel for a given lattice
  protected void update (FiniteLattice lattice)
  {
    // before updating remove all the old stuff
    getContentPane ().removeAll ();

    JPanel all = new JPanel ();
    all.setLayout (new BorderLayout ());
    ((BorderLayout) all.getLayout ()).setHgap (StandardFiller.getSize ());
    ((BorderLayout) all.getLayout ()).setVgap (StandardFiller.getSize ());    
    all.setBorder (StandardFiller.makeWideEmptyBorder ());

    // initialize a tabbed panel
    tabs = new JTabbedPane ();
    
    // check whether negation, meet, and join tables exist
    // for this lattice;
    // if so, create a corresponding table;
    // ow, output a message

    // 1 - meet tab
    Component meet;
    try
    {
      lattice.getMeetTable();
      meet = new JScrollPane (new LatticeTable (lattice,
						FiniteLattice.MEET));
    }
    catch (Exception e)
    {
      meet = new JLabel ("No meet table available", SwingConstants.CENTER);
    }

    tabs.addTab("Meet", new ImageIcon (meetIcon),
		meet, "Display meet table");
    tabs.setSelectedIndex(0);


    // 2 - join tab
    Component join;
    try
    {
      lattice.getJoinTable();
      join = new JScrollPane (new LatticeTable (lattice,
						FiniteLattice.JOIN));
    }
    catch (Exception e)
    {
      join = new JLabel("No join table available",SwingConstants.CENTER);
    }

    tabs.addTab("Join", new ImageIcon (joinIcon),
		join, "Display join table");


    // 3 - negation tab
    Component neg;
    if (lattice.getNegTable() == null)
      neg = new JLabel ("No negation table available",
			SwingConstants.CENTER);
    else
      neg = new JScrollPane (new LatticeTable (lattice,
					       FiniteLattice.NEG));

    tabs.addTab("Negation", new ImageIcon (negIcon),
		neg, "Display negation table");

    // 4 - Colours tab
    JTable colTable = new LatticeTable (new LatticeElementListModel (lattice)
					.addProperty 
					(PrettyConstants.singletonMap, 
					 "Assigned Colour:"));
    colTable.setDefaultRenderer (Color.class, new ColourCellRenderer ());
    colTable.setDefaultEditor (Color.class, new ColourCellEditor ());
    Component colours = new JScrollPane (colTable);
    tabs.addTab ("Colouring", new ImageIcon (coloursIcon),
		 colours, "Edit lattice colouring");


    // initialize the image panel
    File imageFile = lattice.getImageFile();
    if (imageFile != null)
      image = new ImagePanel (Toolkit.getDefaultToolkit().getImage
			      (imageFile.getPath()));
    else
      image = new ImagePanel (null);

    // the close button
    close = new JButton ("Close");
    close.setMaximumSize (new Dimension (50, 50));
    close.setMnemonic (KeyEvent.VK_C);
    close.addActionListener (new ActionListener ()
                               {
				  public void actionPerformed (ActionEvent e)
				    {
				      setVisible(false);
				    }
				});


    // add the components to the panel
    tabs.setPreferredSize (new Dimension (400, 200));
    all.add(tabs, BorderLayout.WEST);
    image.setPreferredSize(new Dimension (200, 200));
    all.add(image, BorderLayout.CENTER);
    all.add(close, BorderLayout.SOUTH);
    
    getContentPane().add(all);
    
    pack();
  }

  public final URL meetIcon =
    getClass().getResource ("/play/meet.gif");
  public final URL joinIcon =
    getClass().getResource ("/play/join.gif");
  public final URL negIcon = 
    getClass().getResource ("/play/neg.gif");
  public final URL coloursIcon = 
    getClass().getResource ("/play/colours.gif");

}// LatticeDisplay

