package edu.toronto.cs.gui;

import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.ctl.*;
import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.tlq.*;
import edu.toronto.cs.smv.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.keg.*;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.prettygraph.*;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;

import javax.swing.table.*;

/**
 * LatticeTables.java
 *
 * An extension of JPanel. Contains a tabbed panel with three tabs:
 * to display negation, meet, and join tables for a particular lattice.
 *
 * Created: Tue Jun 25 11:45:09 2002
 *
 * @author <a href="mailto:anya@tallinn.cs">Tafliovich Anya</a>
 * @version
 */

public class LatticeTables extends JPanel {

  // the tabbed panel
  JTabbedPane tabs;
  // the image panel  
  ImagePanel image;
  

  public LatticeTables (FiniteLattice lattice)
  {
    super();
    init (lattice);
  }

  // initialize a tabbed panel for a given lattice
  private void init (FiniteLattice lattice)
  {
    // initialize a tabbed panel
    tabs = new JTabbedPane ();
    
    // 1 - meet tab
    Component meet = new JScrollPane (new LatticeTable (lattice,
							FiniteLattice.MEET));
    tabs.addTab("Meet", new ImageIcon (meetIcon),
		meet, "Display meet table");
    tabs.setSelectedIndex(0);


    // 2 - join tab
    Component join = new JScrollPane (new LatticeTable (lattice, 
							FiniteLattice.JOIN));
    tabs.addTab("Join", new ImageIcon (joinIcon),
		join, "Display join table");


    // 3 - negation tab
    Component neg = new JScrollPane (new LatticeTable (lattice,
						       FiniteLattice.NEG));
    tabs.addTab("Negation", new ImageIcon (negIcon),
		neg, "Display negation table");

    // 4 - Colours tab
    JTable colTable = new LatticeTable (new LatticeElementListModel (lattice)
					.addProperty 
					(PrettyConstants.singletonMap, 
					 "Assigned Colour:"));
    colTable.setDefaultRenderer (Color.class, new ColourCellRenderer ());
    colTable.setDefaultEditor (Color.class, new ColourCellEditor ());
    Component colours = new JScrollPane (colTable);
    tabs.addTab ("Colouring", new ImageIcon (coloursIcon),
		 colours, "Edit lattice colouring");



    // initialize the image panel
    image = new ImagePanel (Toolkit.getDefaultToolkit().getImage
      // (lattice.getImageFile().getPath());
      ("play/gxl/4val.gif"));
    image.setAlignmentX(Component.CENTER_ALIGNMENT);
    image.setAlignmentY(Component.CENTER_ALIGNMENT);

    // add the tabbed pane to the panel
    setLayout(new GridLayout(1, 1)); 
    add(tabs, BorderLayout.WEST);
    add(image, BorderLayout.EAST);
    setVisible(true);
    
  }

  public final URL meetIcon =
    getClass().getResource ("/toolbarButtonGraphics/navigation/Up24.gif");
  public final URL joinIcon =
    getClass().getResource ("/toolbarButtonGraphics/navigation/Down24.gif");
  public final URL negIcon = 
    getClass().getResource ("/toolbarButtonGraphics/general/Undo24.gif");
  public final URL coloursIcon =
    getClass().getResource ("/toolbarButtonGraphics/general/Undo24.gif");

}// LatticeTables
