package edu.toronto.cs.gui;

import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.ctl.*;
import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.tlq.*;
import edu.toronto.cs.smv.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.keg.*;
import edu.toronto.cs.util.dgraph.*;

import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;

/**
 * ModelOptions.java
 *
 *
 * Created: Wed Jul  3 21:09:06 2002
 *
 * @author <a href="mailto:anya@tallinn.cs">Tafliovich Anya</a>
 * @version
 */

public class ModelOptions extends JDialog {

  // the radio buttons for the various decision diagrams
  JRadioButton mDD;
  JRadioButton mBTDD;
  JRadioButton jBDD;
  JRadioButton cUBDD;
  JRadioButton jADD;
  JRadioButton cUADD;
  JRadioButton cUBiADD;

  // groups the above buttons
  ButtonGroup group;
  
  // an information panel: contains explanations
  // about MvSet options
  JTextArea info;
  
  // gives an option of having state names in a model
  JCheckBox useStateNames;

  // Maps an action command of a radio button to an
  // appropriate explanation string
  HashMap map;
  
  // an "OK" button
  JButton ok;
  
  Map options;
  
  public ModelOptions (Frame owner, Map opts) {
    super (owner, "Select model");
    setModal(true);
    options = opts;
    init ();
  }

  private void init ()
  {
      // make the layout a bit nicer (more space between everything)
    JPanel all = new JPanel ();
    all.setLayout (new BorderLayout ());
    ((BorderLayout) all.getLayout ()).setHgap (StandardFiller.getSize ());
    ((BorderLayout) all.getLayout ()).setVgap (StandardFiller.getSize ());    
    all.setBorder (StandardFiller.makeWideEmptyBorder ());

    map = new HashMap();
    
    // initialize the radio buttons
    mDD = new JRadioButton ("MDD");
    mDD.setMnemonic (KeyEvent.VK_M);
    mDD.setActionCommand ("MDDMvSet");
    map.put("MDDMvSet", mDDInfo);
    // this one is the default one
    mDD.setSelected(true);
    options.put("mvset", "MDDMvSet");

    mBTDD = new JRadioButton ("MBTDD");
    mBTDD.setMnemonic (KeyEvent.VK_T);
    mBTDD.setActionCommand ("JMvSet");
    map.put("JMvSet", mBTDDInfo);

    jBDD = new JRadioButton ("BDD (XChek)");
    jBDD.setMnemonic (KeyEvent.VK_B);
    jBDD.setActionCommand ("JBDDMvSet");
    map.put("JBDDMvSet", jBDDInfo);
    
    cUBDD = new JRadioButton ("BDD (CUDD)");
    cUBDD.setMnemonic (KeyEvent.VK_C);
    cUBDD.setActionCommand ("CUBDDMvSet");
    map.put("CUBDDMvSet", cUBDDInfo);
    
    jADD = new JRadioButton ("ADD (XChek)");
    jADD.setMnemonic (KeyEvent.VK_A);
    jADD.setActionCommand ("JADDMvSet");
    map.put("JADDMvSet", jADDInfo);

    cUADD = new JRadioButton ("ADD (CUDD)");
    cUADD.setMnemonic (KeyEvent.VK_U);
    cUADD.setActionCommand ("CUADDMvSet");
    map.put("CUADDMvSet", cUADDInfo);
    
    cUBiADD = new JRadioButton ("BiADD (CUDD)");
    cUBiADD.setMnemonic (KeyEvent.VK_I);
    cUBiADD.setActionCommand ("CUBiADDMvSet");
    map.put("CUBiADDMvSet", cUBiADDInfo);
    
    // group the radio buttons
    group = new ButtonGroup();
    group.add(mDD);
    group.add(mBTDD);
    group.add(jBDD);
    group.add(cUBDD);
    group.add(jADD);
    group.add(cUADD);
    group.add(cUBiADD);

    // register a listener for the radio buttons
    addRadioListener (mDD);
    addRadioListener (mBTDD);
    addRadioListener (jBDD);
    addRadioListener (cUBDD);
    addRadioListener (jADD);
    addRadioListener (cUADD);
    addRadioListener (cUBiADD);

    // put the radio buttons in a column in a panel
    JPanel radioPanel = new JPanel();
    radioPanel.setLayout(new GridLayout(0, 1));
    radioPanel.add(mDD);
    radioPanel.add(mBTDD);
    radioPanel.add(jBDD);
    radioPanel.add(cUBDD);
    radioPanel.add(jADD);
    radioPanel.add(cUADD);
    radioPanel.add(cUBiADD);

    // set up the information label
    info = new JTextArea(mDDInfo);
    info.setPreferredSize(new Dimension(190, 120));
    info.setFont(new Font (info.getFont().getFontName(), Font.PLAIN, 12));

    // the south panel:
    // check box
    useStateNames = new JCheckBox("use state names");
    useStateNames.setMnemonic(KeyEvent.VK_U);
    useStateNames.setSelected(false);

    useStateNames.addItemListener (new ItemListener ()
      {
	public void itemStateChanged (ItemEvent e)
	{
	  if (e.getStateChange() == ItemEvent.SELECTED)
	    options.put("statenames", new Boolean (true));
	  else
	    options.put("statenames", new Boolean (false));
	}
      });

    // ok button
    ok = new JButton ("OK");
    ok.setMaximumSize (new Dimension (50, 50));
    ok.setMnemonic (KeyEvent.VK_O);
    ok.setSelected(true);
    ok.addActionListener (new ActionListener ()
      {
	public void actionPerformed (ActionEvent e)
	{
	  setVisible(false);
	}
      });

    JPanel south = new JPanel();
    south.setLayout (new BorderLayout ());
    ((BorderLayout) south.getLayout ()).setHgap (StandardFiller.getSize ());
    ((BorderLayout) south.getLayout ()).setVgap (StandardFiller.getSize ());  
    south.setBorder (StandardFiller.makeWideEmptyBorder ());
    south.add(useStateNames, BorderLayout.CENTER);
    south.add(ok, BorderLayout.EAST);

    // add the components
    all.add(radioPanel, BorderLayout.WEST);
    all.add(info, BorderLayout.CENTER);
    all.add(south, BorderLayout.SOUTH);

    getContentPane ().add (all);
    pack();
  }
  
  // register a listener for the radio button
  private void addRadioListener (JRadioButton button)
  {
    button.addActionListener (new ActionListener ()
      {
	public void actionPerformed(ActionEvent e) 
	{
	  info.setText((String)map.get(e.getActionCommand()));
	  options.put("mvset", e.getActionCommand());
	}
      });
  }

  private static String mDDInfo =
    "Multi-valued \nDecision Diagram\n";
  private static String mBTDDInfo =
    "Multi-valued Binary Terminal\nDecision Diagram\n";
  private static String jBDDInfo =
    "Binary Decision Diagram \n(XChek implementation)\n";
  private static String cUBDDInfo = 
    "Binary Decision Diagram \n(CUDD implementation)\n";
  private static String jADDInfo = 
    "Algebraic Decision Diagram \n(XChek implementation)\n";
  private static String cUADDInfo =
    "Algebraic Decision Diagram \n(CUDD implementation)\n";
  private static String cUBiADDInfo = 
    "BiADD \n(CUDD implementation)\n";

}// ModelOptions
