package edu.toronto.cs.keg;

import java.util.*;
import java.io.*;

import edu.toronto.cs.algebra.*;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.ctl.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.heuristics.*;
import edu.toronto.cs.modelchecker.*;

public class KEFairnessSimplifier implements KEGExpander
{

  HeuristicAnalyser heuristics = null;

  public KEFairnessSimplifier ()
  {
  }
  
  
  /**
   ** Get the heuristics.
   ** @return value of heuristics.
   **/
  public HeuristicAnalyser getHeuristics ()
  {
    return heuristics;
  }
  
  /**
   ** Set the heuristics.
   ** @param h -- heuristics to be used.
   **/
  public void setHeuristics(HeuristicAnalyser h)
  {
    heuristics = h;
  }
  
  /**
   ** Attempts to expand the state or the formula associated with the
   ** state in order to provide a counter-example.
   **
   ** @param state -- a node in the state graph that contains the state
   ** to be considered.
   **
   ** @param formula -- a node in the formula graph that contains the
   ** formula to be considered.
   **
   ** @param mc -- the model checker to be used.
   **
   ** @return -- true if the expansion was successful; false otherwise.
   **/
  public boolean expand (DGNode state, DGNode formula, MvSetModelChecker mc)
  {
    // Firstly, get rid of the equals appendage:
    CTLNode ctl = ((KEGCtlNode)formula).getCTL ();
    if (ctl.getClass () != CTLEqualsNode.class)
      return false;

    CTLLatticeNode value = (CTLLatticeNode) ctl.getRight ();
    ctl = ctl.getLeft ();
    
    if (ctl.getFairness ().size () == 0)
      return false;
    
    // Otherwise, split off the fairness condition.
    CTLNode newctl = null;
    //CTLFactory f = ctl.getFactory ();
    
    
    // -- XXX how else to get an MvSet factory?
    MvSetFactory factory = mc.checkCTL (value).getFactory ();
    

    CTLNode phi = ctl.getRight ();

    if (ctl instanceof CTLEXNode)
      newctl = phi.and (createCTL (ctl.getFairness (), mc)).ex ();
    
	//f.createCTLEXNode (f.createCTLAndNode 
	//		 (phi,
	//		  createCTL (ctl.getFairness ())));
    if (ctl instanceof CTLEUNode)
      newctl = 
      //f.createCTLEUNode (ctl.getLeft (), f.createCTLAndNode 
      //			 (phi,
      //
      //createCTL (ctl.getFairness ())));
	ctl.getLeft ().eu (phi.and (createCTL (ctl.getFairness (), mc)));
			   
    // This is the fun one.
    if (ctl instanceof CTLEGNode)
      newctl = 
	CTLFactory.createCTLOrNode 
	(createEGSpecific (phi, (KEGStateNode) state, ctl.getFairness (), factory),
	 CTLFactory.createCTLAndNode 
	 (phi, CTLFactory.createCTLEXNode 
	  (CTLFactory.createCTLEGNode 
	   (CTLFactory.createCTLAndNode 
	    (phi, CTLFactory.createCTLNegNode
	     (CTLFactory.createCTLMvSetNode 
	      (((KEGStateNode)state).getState ()
	       .toMvSet (factory))
	      )), ctl.getFairness ()))));
    if (newctl == null)
      return false;
    
    KegUtils.addCtl (state, (KEGCtlNode) formula, newctl);
     
    return true;
  }

  protected CTLNode createCTL (List l, MvSetModelChecker mc)
  {
    CTLNode top = CTLFactory.createCTLLatticeNode (mc.getLattice ()
						      .idtimesIdx (),
						   mc.getLattice ());
    return CTLFactory.createCTLEGNode (top, l);  
  }

  protected CTLNode createEGSpecific (CTLNode n, KEGStateNode state, List l, MvSetFactory f)
  {
    CTLNode phi = n;
    n = n.eu (CTLFactory.createCTLMvSetNode (state.getState ().toMvSet (f)));
    n = n.ex ();

    for (Iterator it = l.iterator (); it.hasNext ();)
      {
	n = phi.eu (phi.and (((CTLNode) it.next ()).and (n))).ex ();
      }

    return phi.and (n);
  }

    //  return CTLFactory.createCTLAndNode (phi,
	// 				CTLFactory.createCTLEXNode 
// 					(CTLFactory.createCTLEUNode 
// 					 (phi, CTLFactory.createCTLMvSetNode
// 					  (state.toMvSet ())));
// 					}
  
}
