package edu.toronto.cs.keg;

import java.util.*;
import java.io.*;

import edu.toronto.cs.algebra.*;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.ctl.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.heuristics.*;
import edu.toronto.cs.modelchecker.*;

/**
 ** This is a wrapper for the KEGExpander interface to allow chains of responsibility to be built.
 **/
public class KESimplifierChainLink
{
  KEGExpander expander;
  KESimplifierChainLink next = null;

  public KESimplifierChainLink (KEGExpander _expander)
  {
    expander = _expander;
  }
  
  public KESimplifierChainLink addInspector (KESimplifierChainLink _next)
  {
    if (next != null)
      next.addInspector (_next);
    else
      next = _next;
    
    return this;
  }

  public KESimplifierChainLink addInspector (KEGExpander e)
  {
    return addInspector (new KESimplifierChainLink (e));
  }

  public boolean runOnCtl (KEGStateNode initNode, KEGCtlNode initCtlNode, 
			MvSetModelChecker mc)
  {
    Set selection;
    if (expander.expand (initNode, initCtlNode, mc))
      {
	//System.out.println ("Simplifier "+expander+" caught and expanded "+initNode+"'s ctl "+initCtlNode);
	return true;
      }
    else if (next != null)
      {
	//System.out.println ("Simplifier "+expander+" failed to expand and passed on "+initNode+"'s ctl "+initCtlNode );
	return next.runOnCtl (initNode, initCtlNode, mc);
      }
    else
      return false;
  }    

}
  
