/*
 * KVStoreStatic.java
 *
 * Created on November 22, 2001, 2:44 PM
 */

package edu.toronto.cs.kegvis;
import java.util.*;
import java.io.InputStream;

import edu.toronto.cs.ctl.*;
import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.xkripke.*;
import edu.toronto.cs.xkripke.XKripke.*;
import edu.toronto.cs.util.Assert;
import edu.toronto.cs.util.KEGClapi;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.keg.*;
import edu.toronto.cs.util.userprompt.NiceReader;
import edu.toronto.cs.algebra.FiniteLattice;

/**
 *
 * @author BW
 */
public class KVStoreStatic implements KVStore {
  CTLNode ctl;
  MvSetModelChecker modelChecker;
  int numctl = 0;
  
  /** options map is stored for no real reason right now */
  Map options;
  
  String filename;
  MvState initState = null;
  XKripke xk;
  
  DGraph KGraph;
  
  
  /** Creates new KVStoreStatic */
  public KVStoreStatic(Map options) {
    this.options = options;
    this.filename = (String) (((List) options.get("input")).get(0));
  }
  
  public KVStoreStatic(String filename) {
    this.filename = filename;
  }
  
  public KVStoreStatic() {
  }
  
  public DGraph getKGraph() {
    return KGraph;
  }
  
  /** An example of how to initialize this, but probably won't take console
   * input in real usage. Actually ignores everything from command line except
   * the xml filename.
   * @param args the command line arguments
   */
  public static void main(String args[]) {
    // parse command line arguments
    Map opts = KEGClapi.parseOpts(args);
    KVStoreStatic testStore = new KVStoreStatic(opts);
    testStore.initModel();
    
    while (true) {
      
      // get some CTL input
      CTLNode ctlinput = null;
      System.out.println("Enter CTL:");
      NiceReader in = new NiceReader(System.in);
      ctlinput = testStore.getCtlFrom(in.readLine());
      
      if (ctlinput == null)
        break;
      
      // place it in the KVStore
      testStore.newCtl(ctlinput);
    }
  }
  
  /** Read CTL string from an InputStream. Call getCtlFrom(String) to parse.
   * @param source InputStream to read CTL from
   * @return the root CTLNode of the parsed CTL
   */
  public CTLNode getCtlFrom(InputStream source) {
    // -- prepare to read from stdin
    NiceReader in = new NiceReader(source);
    String ctlStr = in.readLine();
    return getCtlFrom(ctlStr);
  }
  
  /** Parse CTL from a string. Must have initialized modelChecker first.
   * @param ctlString the String of CTL to parse
   */
  public CTLNode getCtlFrom(String ctlString) {
    if (!(ctlString == null) && !ctlString.equals("")) {
      BinaryTreeToCTLConverter.AssignLattice(modelChecker.getLattice());
      CTLNode ctl = BinaryTreeToCTLConverter.convertToCTL(ctlString);
      return ctl;
    }
    else
      return null;
  }
  
  /** initialize the model checker using the default XKripke found in the XML
   * file from the options Map.
   */
  public void initModel() {
    if (filename != null)
      initModel(filename);
  }
  
  /**
   * initModel(String) will initialize the model checker using the
   * XKripke found in the XML file specified.
   */
  public void initModel(String filename) {
    this.filename = filename;
    
    // try to parse xml XKripke and instantiate a model checker
    try {
      xk = XKripkeFactory.parse(filename);
    }
    catch (Exception e) {
      Assert.assert(e);
    }
    
    initModel(xk);
  }
  
  /**
   * initModel(XKripke) will initialize the model checker using the
   * XKripke directly passed in
   */
  public void initModel(XKripke xk) {
    try {
      //check if states should be named (what does this do?)
      //	  if (((Boolean) options.get("statenames")).booleanValue())
      //		xk.labeliseStates();
      modelChecker = new MvSetModelChecker(xk);
    }
    catch (RuntimeException e) {
      Assert.assert(e);
    }
    
    //set the initial state
    XKripkeState[] states = xk.getInitialStates();
    if (states.length < 1) {
      System.out.println("No initial states!");
    }
    
    else { // at least one initial state
      initState = new MvState(states[0], modelChecker.getPropOrdering());
      if (states.length > 1)
        System.out.println("WARNING: No unique initial state.");
    }
  }
  
  /** Rebuilds the KGraph using the new CTL.
   *
   * @param ctl the CTL to expand
   */
  public void newCtl(CTLNode ctl) {
    this.ctl = ctl;
    numctl++;
    
    KGraph = initKegStruct(ctl, initState);
    KEGStateNode initNode =
    (KEGStateNode)KGraph.getRootNodes().iterator().next();
    KEGCtlNode initCtlNode =
    (KEGCtlNode)initNode.getCTLGraph().getRootNodes().iterator().next();
    
    // now recursively expand the ctl
    runOnCtl(initNode, initCtlNode);
  }
  
  protected void runOnCtl(KEGStateNode initNode, KEGCtlNode initCtlNode) {
    // new ctlsimplifier-type kegexpander with no heuristics
    KEGExpander ctlexp = new KECTLSimplifier();
    KEGExpander statexp = new KEMDDStateGenerator();
    Set selection;
    
    // first try to expand ctl
    if (ctlexp.expand(initNode, initCtlNode, modelChecker)) {
      // look to see what new successor ctl came out of it
      selection = new HashSet(initCtlNode.getSuccessors());
      
      // now recursively expand all new ctl branches
      for (Iterator it = selection.iterator(); it.hasNext();) {
        runOnCtl(initNode, (KEGCtlNode) it.next());
      }
    }
    // ctl expansion unsuccessful: (EX?) try state expansion
    else if (statexp.expand(initNode, initCtlNode, modelChecker)) {
      // look to see what new successor states came out of it
      selection = new HashSet(initNode.getSuccessors());
      
      // now recursively expand all new states
      for (Iterator it = selection.iterator(); it.hasNext();) {
        // let's just make this easier to read:
        KEGStateNode nextstate = (KEGStateNode) it.next();
        KEGCtlNode nextctl = (KEGCtlNode)
        nextstate.getCTLGraph().getRootNodes().iterator().next();
        runOnCtl(nextstate, nextctl);
      }
    }
    //else it was terminal, so done this recursion
  }
  
  protected DGraph initKegStruct(CTLNode ctl, MvState initState) {
    DGraph KGraph = new DGraph(new KEGStateNodeFactory());
    DGraph ctlGraph = new DGraph(new KEGCtlNodeFactory());
    
    KEGStateNode stateNode =
    ((KEGStateNodeFactory) KGraph.getFactory()).createNode(null, initState);
    
    KGraph.addRootNode(stateNode);
    
    stateNode.setCTLGraph(ctlGraph);
    
    if (ctl != null) {
      KEGCtlNode ctlNode =
      ((KEGCtlNodeFactory) ctlGraph.getFactory()).createNode(null,ctl);
      ctlGraph.addRootNode(ctlNode);
    }
    
    return KGraph;
  }
  
  /**
   * should somehow dump some ugly representation of the graph
   * for debugging purposes (but isn't done)
   */
  void dumpKEGStateGraph() {
    System.out.println();
  }
  
  public PropOrderIf getPropOrder() {
    return modelChecker.getPropOrdering();
  }
  
  public FiniteLattice getLattice() {
    return modelChecker.getLattice();
  }
  
  /** Getter for property filename.
   * @return Value of property filename.
   */
  public java.lang.String getFilename() {
    return filename;
  }
  
  /** Setter for property filename.
   * @param filename New value of property filename.
   */
  public void setFilename(java.lang.String filename) {
    this.filename = filename;
  }
  
  /** Getter for property numctl.
   * @return Value of property numctl.
   */
  public int getNumctl() {
    return numctl;
  }
  
}

