/*
 * KVWSession.java
 *
 * Created on November 22, 2001, 11:42 AM
 */

package edu.toronto.cs.kegvis;
import java.util.*;
import edu.toronto.cs.util.KEGClapi;
import edu.toronto.cs.ctl.CTLNode;
import edu.toronto.cs.keg.*;
/**
 *
 * @author  bwong
 * @version
 */
public class KVWSession implements java.io.Serializable {
  
  KVStoreStatic store;
  
  /** the root CTL for this session */
  String ctlString;
  
  String filename;
  
  KVWCtlFilter ctlfilter;
  
  KVWPropFilter propfilter;
  
  KVWStateFilter statefilter;
  
  /** stores the choice of whether to step "left" or "right" in a CTL branch */
  int choice = 0;
  
  /** Vector of KVStates */
  Vector states;
  
  /** Creates new KVWSession */
  public KVWSession(String filename) {
    // the initial model
    this.filename = filename;
    
    if (filename != null) {
      store = new KVStoreStatic(filename);
      store.initModel();
    } else {
      store = new KVStoreStatic();
    }
    states = new Vector();
    ctlfilter = new KVWCtlFilter();
    propfilter = new KVWPropFilter();
    statefilter = new KVWStateFilter();
  }
  
  /** Creates a new session with an uninitialized KVStore. Will need to call
   * setFilename and setCtl to use.
   */
  public KVWSession() {
    store = new KVStoreStatic();
    states = new Vector();
    ctlfilter = new KVWCtlFilter();
    propfilter = new KVWPropFilter();
    statefilter = new KVWStateFilter();
  }
  
  /** Adds one step, if possible.
   * @return the number of steps successfully made (0 or 1)
   */
  public int addSteps() {
    // if there are no steps yet, get first state
    if (states.isEmpty()) {
      
      if (store.getKGraph().getRootNodes().size() == 0) {
        //uh oh
        System.out.println("KVWSession: Counterexample graph is empty!");
        return 0;
      }
      
      // TODO: make sure there is only one root? Should be..
      KEGStateNode statenode =  
      (KEGStateNode)store.getKGraph().getRootNodes().iterator().next();
      states.add(new KVWState(statenode, store.getPropOrder()));
    }
    else {
      // there are states, so add ctl if possible, or states if not
      
      KVWState laststate = (KVWState) states.lastElement();
      // should have an element... maybe check
      KVWCtlExpression lastctl =
      (KVWCtlExpression) laststate.getCtl().lastElement();
      
      if (lastctl.numSuccessors() > 0) {
        // new ctl needed
        if (lastctl.numSuccessors() == 1) {
          // good, just add it
          laststate.getCtl().add(new KVWCtlExpression((KEGCtlNode)lastctl.getSuccessors().get(0)));
        }
        else if (lastctl.numSuccessors() == 2) {
          // choice: left (1) or right (0) side of /\ or \/
          laststate.getCtl().add(new KVWCtlExpression((KEGCtlNode)lastctl.getSuccessors().get(choice)));
        }
        else {
          // What happened here?
          throw new RuntimeException("What could cause more than 2 CTL successors?");
        }
      }
      else {
        // new state needed?
        Vector successors = laststate.getSuccessors();
        if (successors.size() == 0) {
          // all done!
          return 0;
        }
        KEGStateNode statenode = (KEGStateNode) successors.remove(0);
        
        states.add(new KVWState(statenode, store.getPropOrder(), successors,
        store.getLattice().getElementKeyByIdx(laststate.getTransValue(statenode))));
      }
    }
    return 1;
  }
  
  public int addSteps(int numsteps) {
    int i = 0;
    for (; i<numsteps; i++) {
      if (addSteps() == 0)
        break;
    }
    return i;
  }
  
  public java.lang.Object clone() {
    KVWSession newSession = new KVWSession(filename);
    newSession.setCtlfilter(ctlfilter);
    newSession.setPropfilter(propfilter);
    newSession.setStatefilter(statefilter);
    return (Object) newSession;
  }
  
  public ListIterator getStateIterator() {
    return states.listIterator();
  }
  
  /** Getter for property ctlfilter.
   * @return Value of property ctlfilter.
   */
  public edu.toronto.cs.kegvis.KVWCtlFilter getCtlfilter() {
    return ctlfilter;
  }
  
  /** Setter for property ctlfilter.
   * @param ctlfilter New value of property ctlfilter.
   */
  public void setCtlfilter(edu.toronto.cs.kegvis.KVWCtlFilter ctlfilter) {
    this.ctlfilter = ctlfilter;
  }
  
  /** Getter for property propfilter.
   * @return Value of property propfilter.
   */
  public edu.toronto.cs.kegvis.KVWPropFilter getPropfilter() {
    return propfilter;
  }
  
  /** Setter for property propfilter.
   * @param propfilter New value of property propfilter.
   */
  public void setPropfilter(edu.toronto.cs.kegvis.KVWPropFilter propfilter) {
    this.propfilter = propfilter;
  }
  
  /** Getter for property statefilter.
   * @return Value of property statefilter.
   */
  public edu.toronto.cs.kegvis.KVWStateFilter getStatefilter() {
    return statefilter;
  }
  
  /** Setter for property statefilter.
   * @param statefilter New value of property statefilter.
   */
  public void setStatefilter(edu.toronto.cs.kegvis.KVWStateFilter statefilter) {
    this.statefilter = statefilter;
  }
  
  /** Getter for property states.
   * @return Value of property states.
   */
  public java.util.Vector getStates() {
    return states;
  }
  
  /** Setter for property states.
   * @param states New value of property states.
   */
  public void setStates(java.util.Vector states) {
    this.states = states;
  }
  
  /** Emulates the servlet for testing.
   * @param args the command line arguments
   */
  public static void main(String args[]) {
    String filename = "play/gxl/3rdmergedphone1.xml";
    String ctlString = "EX (offhook \\/ calleesel)";
    System.out.println("Using [" + ctlString + "] on " + filename);
    //    KVWSession kvsession = new KVWSession("play/gxl/coffee1.xml");
    //    kvsession.setCtlString("EX((power = DN) /\\ (cup = DC))");
    KVWSession kvsession = new KVWSession(filename);
    kvsession.setCtlString(ctlString);
    
    System.out.println("Steps: " + kvsession.addSteps(6));
    
    System.out.println("<table>");
    for (ListIterator sit = kvsession.getStateIterator(); sit.hasNext();) {
      KVWState state = (KVWState) sit.next();
      
      // just transitioned?
      if (sit.previousIndex() > 0) {
        System.out.println("<tr><td><!-- (begin transition) -->");
        System.out.println(state.getTransValue());
        System.out.println("</td><td>");
        if (state.getNeighbours().size() > 0) {
          System.out.println("Other states needed: ");
          for (Iterator asit = state.getNeighbours().iterator(); asit.hasNext();) {
            KEGStateNode altState = (KEGStateNode) asit.next();
            System.out.println("<a href=\"#\">" + altState.toString() + "</a>");
          }
        }
        System.out.println("</td></tr><!-- (end transition) -->");
      }
      
      if (kvsession.getStatefilter().isDisplayable(state)) {
        System.out.println("<tr><!-- (begin staterow) -->");
        System.out.println("<td><!-- (begin display state) -->");
        System.out.println("<table>");
        if (state.isLabelized()) {
          System.out.println("<tr><td><!-- (begin label) -->");
          System.out.println(state.getLabel());
          System.out.println("</td></tr><!-- (end label) -->");
        }
        for (Iterator pit = state.getPropIterator(); pit.hasNext();) {
          KVWStateProperty prop = (KVWStateProperty) pit.next();
          if (kvsession.getPropfilter().isDisplayable(prop)) {
            System.out.println("<tr><td><!-- (begin prop) -->");
            System.out.println(prop.getName());
            System.out.println("</td><td>");
            System.out.println(prop.getValue());
            System.out.println("</td></tr><!-- (end prop) -->");
          } else {
            System.out.println("<!-- skip prop -->");
          }
        }
        System.out.println("</table></td><!-- (end display state) -->");
        
        // now the CTL side
        System.out.println("<td><table><!-- (begin display CTL) -->");
        // maybe show a header row here
        for (Iterator cit = state.getCtl().iterator(); cit.hasNext();) {
          KVWCtlExpression ctlex = (KVWCtlExpression) cit.next();
          if (kvsession.getCtlfilter().isDisplayable(ctlex)) {
            System.out.println("<tr><td><!-- (begin ctl) -->");
            System.out.println(ctlex.toHtml());
            System.out.println("</td><td>");
            System.out.println(ctlex.nextOp());
            System.out.println("</td></tr><!-- (end ctl) -->");
          } else {
            System.out.println("<!-- skip ctlex -->");
          }
        }
        System.out.println("</table></td><!-- (end display CTL) -->");
        System.out.println("</tr><!-- (end staterow) -->");
      } else {
        System.out.println("<!-- skip state -->");
      }
    }
    System.out.println("</table><!-- (all done) -->");
  }
  
  /** Getter for property ctlString.
   * @return Value of property ctlString.
   */
  public java.lang.String getCtlString() {
    return ctlString;
  }
  
  /** Setter for property ctlString. Causes regeneration of counter-example
   * @param ctlString New value of property ctlString.
   */
  public void setCtlString(java.lang.String ctlString) {
    this.ctlString = ctlString;
    
    store.newCtl(store.getCtlFrom(ctlString));
    addSteps(1); // try to get the first step out there
  }
  
  /** Getter for property choice.
   * @return Value of property choice.
   */
  public int getChoice() {
    return choice;
  }
  
  /** Setter for property choice.
   * @param choice New value of property choice.
   */
  public void setChoice(int choice) {
    this.choice = choice;
  }
  
  /** resets the trace without regenerating counter-example */
  public void resetTrace() {
    states.clear();
    addSteps(1);
  }
  
  /** Getter for property filename.
   * @return Value of property filename.
   */
  public java.lang.String getFilename() {
    return filename;
  }
  
  public void setFilename(java.lang.String filename) {}
  
}
