package edu.toronto.cs.lift;

import edu.toronto.cs.smv.*;
import edu.toronto.cs.mvset.*;

import edu.toronto.cs.algebra.FiniteLattice;

public class ButtonRAbstract extends SMVModule
{
  static final int PRESSED = 0;
  static final int BUTTON = 1;
  
  MvSet reset = null;

  MvSet pressedCond = null;
  int pressedCondVal = -1;

  int maybe;

  public ButtonRAbstract (String _name)
  {
    super (_name);
    vars = new String[]{"pressed", "button"};
  }
  

  public void setLattice (FiniteLattice lattice)
  {
    super.setLattice (lattice);
    maybe = lattice.getElementIdxByKey ("M");
  }

  public void setReset (MvSet v)
  {
    reset = v;
  }
  
  public void setPressedCond (MvSet cond, int val)
  {
    pressedCond = cond;
    pressedCondVal = val;
  }
  
//    public MvSet getPressed ()
//    {
//      if (pressedCond == null)
//        return cur (PRESSED);
    
//      return new CaseStatement ().
//        addCase (pressedCond.and (cur (PRESSED)), cur (PRESSED, bot)).
//        addDefault (cur (PRESSED)).compute ();
    
    
//    }
  
  public MvSet getPressed ()
  {
    return cur (PRESSED);
  }
  public MvSet getButton ()
  {
    return cur (BUTTON);
  }
  
  
  public MvSet computeTrans ()
  {
    System.out.println ("Computing trans for: " + getName ());
    return computeButton ().and (computePressed ());
  }
  

  public MvSet computeButton ()
  {
    /*** the current value of the button is 'maybe' if either reset
     *** of pressed are true, in all other cases it is chosen non
     *** determenistically
     ***/
    
    /***
     *** button := case
     ***            pressed : maybe;
     ***            reset   : maybe;
     ***            1       : {true, false};
     ***           esac;
     ***/

    MvSet button = new CaseStatement ().
      addCase (cur (PRESSED), cur (BUTTON, maybe)).
      addCase (reset, cur (BUTTON, maybe)).
      addDefault (cur (BUTTON).or (cur (BUTTON, bot))).compute ();
    
    
    // -- now create the next state representation
    button = button.and (button.renameArgs (ordering.getPrimedMap ()));
    return button;
//      return new CaseStatement ().
//        addCase (cur (PRESSED), next (BUTTON, maybe)).
//        addCase (reset, next (BUTTON, maybe)).
//        addDefault (next (BUTTON).or (next (BUTTON, bot))).compute ();
  }
  
  public MvSet computePressed ()
  {

    /***
	next (pressed) := case
	                    reset  : false;
			    button : true;
			    1      : pressed;
    ***/			   
    MvSet requestIsRequest = 
      ddManager.createProjection (getNextVarOrder (PRESSED)).
      eq (ddManager.createProjection (getVarOrder (PRESSED)));

    return new CaseStatement ().
      addCase (reset, next (PRESSED, bot)).
      addCase (cur (BUTTON), next (PRESSED)).
      addDefault (requestIsRequest).compute ();
  }
  

  public MvState initializeInitState (MvState initState)
  {
    // -- init (pressed) := false
    initState.setValue (getVarOrder (PRESSED), bot);
    // -- init (button) := false
    initState.setValue (getVarOrder (BUTTON), bot);
    return initState;
  }
  
    
}
