package edu.toronto.cs.lift;

import edu.toronto.cs.smv.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.mdd.*;
import edu.toronto.cs.algebra.*;

import edu.toronto.cs.util.*;

import java.util.Arrays;

public class MainLiftRAbstract extends SMVCombinedModule
{
  ButtonRAbstract[] landingBtns;
  
  LiftRAbstract lift;

  public MainLiftRAbstract (String name)
  {
    super (name);
  }
  
  public MainLiftRAbstract (String name, int floors)
  {
    super (name);

    lift = new LiftRAbstract ("lift", floors, this);
    landingBtns = new ButtonRAbstract [floors];

    for (int i = 0; i < floors; i++)
      landingBtns [i] = new ButtonRAbstract ("landingBut" + (i+1));
    
    addSubModules (Arrays.asList (landingBtns));
    addSubModule (lift);
  }
  
  public ButtonRAbstract[] getLandingBtns ()
  {
    return landingBtns;
  }
  
  public MvSet computeTrans ()
  {
    MvSet result = computeButtons ().and (computeLift ());
//      if (result instanceof MDDMvSet)
//        result = boolResult ((MDDMvSet)result);
    return result;
  }

  MvSet boolResult (MDDMvSet set)
  {
    MDDMvSet.MDDMvSetFactory mddFactory = 
      (MDDMvSet.MDDMvSetFactory)set.getFactory ();
    
    MDDManager mddManager = mddFactory.getMddManager ();


    MDDNode node = set.getMddNode ();

    int size = ordering.size ();
    boolean[] map = new boolean [size];
    Arrays.fill (map, false);
    for (int i = 0; i < size; i++)
      {
	String name = ordering.getPropName (i);
	if (name.endsWith ("button"))
	  map [i] = true;
      }
    MDDNode result = 
      mddManager.killEdge (node, map, lattice.getElementIdxByKey ("M"));
    
    return mddFactory.createMvSet (result);
    
    
  }
  
  
  MvSet computeLift ()
  {
    lift.setNoCall (computeNoCall ());
    return lift.computeTrans ();
  }
  
  MvSet computeButtons ()
  {
    MvSet result = mvTop;
    
    MvSet doorIsOpen = lift.cur (LiftRAbstract.DOOR, top);

    // -- for each button initialize it using
    // -- landingBut_i : button (lift.floor = i & lift.door = open)
    for (int i = 0; i < landingBtns.length; i++)
      {
	landingBtns [i].setReset 
	  (lift.computeFloorEquals (i+1).and (doorIsOpen));
	result = result.and (landingBtns [i].computeTrans ());
      }
    return result;
  }
  
  MvSet computeNoCall ()
  {
    MvSet result = mvTop;

    // -- no_call := !landingBut1.pressed & !landingBut2.pressed & ...

    for (int i = 0; i < landingBtns.length; i++)
      result = result.and (landingBtns [i].getPressed ().not ());
    return result;
    
  }
  
  
  MvSet computeLandingCall ()
  {
    return null;
  }
  
  
  public MvState initializeInitState (MvState initState)
  {
    super.initializeInitState (initState);
    return initState;
  }


  // -- Module intialization routine to be used by the model checker
  public static SMVModule initModule (String[] args)
  {
    System.out.println ("Intializing: " + MainLiftRAbstract.class.getName ());
    System.out.println ("Will refer to this module as \'main\'");
    String moduleName = "main";
    System.out.println ("Expecting one argument: numer of floors");

    int floors;
    try {
      floors = Integer.parseInt (args [0]);
    } catch (Exception ex) {
      System.out.println ("Hmm no arguments, got: " + ex);
      System.out.println ("Assuming 3 floors");
      floors = 3;
    }
    
    
    String latticeFile = "play/gxl/3val-logic.xml";
    FiniteLattice lattice = null;
    System.out.println ("Using " + latticeFile + " as our lattice");
    try {
      lattice = LatticeFactory.parse (latticeFile);
    } catch (Exception ex) {
      System.out.println ("Failed to parse: " + latticeFile);
      ex.printStackTrace ();
      Assert.assert ("Failed to parse the lattice", ex);
    }
    
    MainLiftRAbstract main = new MainLiftRAbstract (moduleName, floors);
    main.setLattice (lattice);
    return main;
  }
  
  
}
