package edu.toronto.cs.lift;

import edu.toronto.cs.smv.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.algebra.*;

import edu.toronto.cs.util.*;

import java.util.Arrays;

public class MainLiftRTT extends SMVCombinedModule
{
  ButtonR[] landingBtns;
  
  LiftRTT lift;
  
  public MainLiftRTT (String name, int floors)
  {
    super (name);

    lift = new LiftRTT ("lift", floors, this);
    landingBtns = new ButtonR [floors];

    for (int i = 0; i < floors; i++)
      landingBtns [i] = new ButtonR ("landingBut" + (i+1));
    
    addSubModules (Arrays.asList (landingBtns));
    addSubModule (lift);
  }
  
  public ButtonR[] getLandingBtns ()
  {
    return landingBtns;
  }
  
  public MvSet computeTrans ()
  {
    return computeButtons ().and (computeLift ());
  }
  
  MvSet computeLift ()
  {
    lift.setNoCall (computeNoCall ());
    return lift.computeTrans ();
  }
  
  MvSet computeButtons ()
  {
    MvSet result = mvTop;
    
    MvSet doorIsOpen = lift.cur (LiftRTT.DOOR, top);

    /*** PressedCondition is a condition when landingBut.pressed should be
     *** treated as false, it is
     *** lift.tt_full & \E{i} lift.liftBut_i.pressed
     ***
     *** That is if lift is two thirds full and a button inside the lift
     *** is pressed, act as though no landing button is pressed 
     ***/
    MvSet pressedCond = mvBot;
    for (int i = 0; i < lift.liftBtns.length; i++)
      pressedCond = pressedCond.or
	(lift.liftBtns [i].getPressed ());

    pressedCond = pressedCond.and (lift.cur (LiftRTT.TTFULL));
    

    // -- for each button initialize it using
    // -- landingBut_i : button (lift.floor = i & lift.door = open)
    for (int i = 0; i < landingBtns.length; i++)
      {
	landingBtns [i].setReset 
	  (lift.computeFloorEquals (i+1).and (doorIsOpen));
	
	// -- FEATURE
	landingBtns [i].setPressedCond (pressedCond, bot);
	
	result = result.and (landingBtns [i].computeTrans ());
      }
    return result;
  }
  
  MvSet computeNoCall ()
  {
    MvSet result = mvBot;

    // -- no_call := !landingBut1.pressed & !landingBut2.pressed & ...

//      for (int i = 0; i < landingBtns.length; i++)
//        result = result.and (landingBtns [i].getPressed ().not ());
    for (int i = 0; i < landingBtns.length; i++)
      result = result.or (landingBtns [i].getPressed ());
    result = result.not ();

    if (result instanceof MDDMvSet)
      {
	((MDDMvSet)landingBtns [0].getPressed ()).
	  writeToDaVinci ("getPressed.daVinci", ordering);
	((MDDMvSet)landingBtns [0].getPressed ().not ()).
	  writeToDaVinci ("notGetPressed.daVinci", ordering);
	((MDDMvSet)result).writeToDaVinci ("nocall.daVinci", ordering);
      }

    return result;
    
  }
  
  
  MvSet computeLandingCall ()
  {
    return null;
  }
  
  
  public MvState initializeInitState (MvState initState)
  {
    super.initializeInitState (initState);
    return initState;
  }


  // -- Module intialization routine to be used by the model checker
  public static SMVModule initModule (String[] args)
  {
    System.out.println ("Intializing: " + MainLiftRTT.class.getName ());
    System.out.println ("Will refer to this module as \'main\'");
    String moduleName = "main";
    System.out.println ("Expecting one argument: numer of floors");

    int floors;
    try {
      floors = Integer.parseInt (args [0]);
    } catch (Exception ex) {
      System.out.println ("Hmm no arguments, got: " + ex);
      System.out.println ("Assuming 3 floors");
      floors = 3;
    }
    
    
    String latticeFile = "play/gxl/boolean.xml";
    FiniteLattice lattice = null;
    System.out.println ("Using " + latticeFile + " as our lattice");
    try {
      lattice = LatticeFactory.parse (latticeFile);
    } catch (Exception ex) {
      System.out.println ("Failed to parse: " + latticeFile);
      ex.printStackTrace ();
      Assert.assert ("Failed to parse the lattice", ex);
    }
    
    MainLiftRTT main = new MainLiftRTT (moduleName, floors);
    main.setLattice (lattice);
    return main;
  }
  
  
}
