package edu.toronto.cs.mdd;

import java.util.*;
import java.io.*;

import edu.toronto.cs.mvset.*;
import edu.toronto.cs.xkripke.XKripke;
import edu.toronto.cs.util.Logger;
import edu.toronto.cs.util.Assert;
import edu.toronto.cs.algebra.*;

import edu.toronto.cs.modelchecker.PropOrderIf;
import edu.toronto.cs.util.dgraph.DGraph;
import edu.toronto.cs.util.dgraph.DGNode;
import edu.toronto.cs.util.dgraph.DGDepthTraverse;
import edu.toronto.cs.prettygraph.PrettyGraphMDDVisitor;
import edu.toronto.cs.prettygraph.DVGenEdgeVisitor;
import edu.toronto.cs.prettygraph.DVGenNodeVisitor;



public class MDDNode 
{
  


  public static long[] primes = {12582917L, 4256249L, 741457L, 1618033999L};
  

  // -- number of children to use for hashing
  public static int hashChildren = 4;

  int variableIndex;
  // int refcount;
  int numChildren = 0;
  int enumerationIndex;
  boolean isConstant;

  boolean visited;

  BitSet image;
  int value; // defined only if constant
  MDDNode[] children; // defined only if not constant

  // -- parent MDD manager
  MDDManager manager;

  //static Logger log = new Logger(MDDNode.class.getName());

  int largestIndex = -1;

  public MDDNode(int vidx, MDDNode[] chdrn)
  {
    throw new RuntimeException 
      ("MDDManager must be passed in the constructor");
  }
  

  public MDDNode(MDDManager _manager, int vidx, MDDNode[] chdrn)
  {
    manager = _manager;
    variableIndex = vidx;
    //numChildren = chdrn.length;
    //children = new MDDNode [numChildren];
    //System.arraycopy(chdrn, 0, children, 0, numChildren);
    //isConstant = false;
    setChildren (chdrn);
    recalcImage ();
  }

  public MDDNode (MDDManager _manager)
  {
    manager = _manager;
  }
  
  public MDDNode(int vidx, int val) 
  {
    throw new RuntimeException 
      ("MDDManager must be passed in the constructor");
  }
  
  /** Constant-node constructor. */
  public MDDNode(MDDManager _manager, int val) 
  {
    manager = _manager;
    isConstant = true;
    variableIndex = Integer.MAX_VALUE;
    value = val;
    image = new BitSet();
    image.set (val);
  }

  public void setValue (int val)
  {
    isConstant = true;
    variableIndex = Integer.MAX_VALUE;
    value = val;
    // -- avoid setting bitsets
  }
  
  public void setChildren (MDDNode[] _children)
  {
    isConstant = false;
    children = _children;
    numChildren = children.length;
  }

  public int getLargestIndex (int val)
  {
    if (largestIndex < 0) 
      largestIndex = getLargestIndexRecur (val);
    return largestIndex;
  }
  
  int getLargestIndexRecur (int val)
  {
    if (getChild (val).isConstant ())
      return getVarIndex ();
    
    return getChild (val).getLargestIndexRecur (val);
  }
  
    
  public static final BitSet EMPTY_BIT_SET = new BitSet ();

  private void recalcImage ()
  {
    // -- This is currently used by counter-example generator
    image = new BitSet (numChildren);
    for (int i = 0; i < numChildren; i++)
      image.or (children [i].image);
    //image = EMPTY_BIT_SET;
  }
  

  public MDDManager getManager ()
  {
    return manager;
  }
  
  // -- return human readable form of the value
  public Object getValueKey ()
  {
    if (!isConstant ()) return null;
    return getManager ().getAlgebra ().getElementKeyByIdx (getValue ());
  }
  
  /** <b>Predicate</b>: has the node been marked visited? */
  public boolean wasVisited() 
  {
    return wasVisited();
  }

  public int howManyChildren() 
  {
    return numChildren;
  }

  public void setVisited(boolean v) 
  {
    visited = v;
  }


  public String toString() 
  {
    String s;
    if (isConstant) 
      s= new String("[MDDNode] Constant: value="+value+"/"+image.toString());
    else
      s = new String("[MDDNode]: Variable #"+variableIndex+"/"+
		     image.toString());
	
    return s;
    
//      if (visited) 
//        return s+"MARKED.";
//      else return s;
	
  }


  /** Check to see if value v is in image */
  public boolean inImage (int v) 
  {
    return image.get(v);
  }

  public BitSet getImage()
    {
      return (BitSet)image.clone();
    }

  public int getVarIndex() 
  {

    return variableIndex;
  }
  public void setVarIndex (int idx)
  {
    variableIndex = idx;
  }
  

  public boolean isConstant() 
  {
    return isConstant;
  }

  public boolean isConstant(int i) 
  {
    return (isConstant && (value==i));
  }

  public int getValue() 
  {
    return value;
  }

  /*** returns 'null' if a node is constant */
  public MDDNode getChild(int i) 
  {
    if (!isConstant ())
      return children [i];
    else
      return null;
  }

  public boolean equals (Object o)
  {
    if (this == o) return true;
    if (o instanceof MDDNode)
      return equals ((MDDNode)o);
    return false;
  }
  public boolean equals (MDDNode node)
  {
    if (variableIndex != node.variableIndex) return false;
    if (isConstant ()) return value == node.value;
    
    for (int i = 0; i < numChildren; i++)
      if (children [i] != node.children [i]) return false;
    return true;
    
  }
  
  
  /***
   * Returns true if n2 is an MDDNode equal to this one.
   */
  public boolean __equals(Object n2) 
  {
    if (n2 == this) return true;
    
    if (n2 instanceof MDDNode)
      {
	
	MDDNode mdd2 = (MDDNode)n2;
	boolean eq = (mdd2.variableIndex == variableIndex);
	if (eq)
	  {
	    if (isConstant ())
	      eq = (mdd2.value == value);
	    else 
	      {
		int i = 0;
		while (i < numChildren && eq) 
		  {
		    eq = (children[i] == mdd2.children[i]);
		    i++; 
		  }
	      }
	  }
	return eq;
      }
    else return false;
  }
  
  // -- pointer based hash code
  public int objectHashCode ()
  {
    return super.hashCode ();
  }
  
  public int hashCode ()
  {
    
    if (isConstant ())
      return (int) (getValue () * primes [0]);

    long hash = 0;
    int len = numChildren > hashChildren ? hashChildren : numChildren;
    
    for (int i = 0; i < len; i++)
      hash += children [i].objectHashCode () * primes [i];
//      hash = children [manager.getOne ().getValue ()].objectHashCode ()
//        * primes [0] + 
//        children[manager.getZero ().getValue ()].objectHashCode () * primes [1];
    return (int)hash;
  }
  
  public int __2hashCode ()
  {
    // XXX What would be a good hashcode?
    int hash = toString ().hashCode ();
    if (isConstant ())
      hash += value;
    else
      hash += children [numChildren - 1].variableIndex;
    return hash;
  }
  
  public int __hashCode() 
  {
    /*** Is this a good hashCode? */
    if (isConstant ())
      return variableIndex;
    else 
      return variableIndex + children [numChildren-1].variableIndex;
  }

  MDDNode restrictMDD (List stateRestriction, MDDNode mdd)
    {
      MDDNode result = mdd;
      int varIndex=0;
      for (Iterator it = stateRestriction.iterator ();
	   it.hasNext (); varIndex++)
	{
	  int varValue = ((Integer)it.next ()).intValue ();
	  result = manager.Restrict (result, varIndex, varValue);
	}
      return result;
    }

  /**
   ** Takes a BitSet specifying which lattice values to find a
   ** restriction for.
   **
   ** @param values -- BitSet showing lattice values in which you are
   ** interested
   **
   ** @return -- Set of restrictions for the given values.
   **/
  public Set findStates (BitSet values)
  {
    //log.debug ("Looking for restrictions for: " + values);
    Set restrictions = new HashSet ();

    // get an MvState for every value
    for (int val = 0; val < values.size (); val++)
      {
	if (values.get (val))
	  {
	    // first create a 'garbage' state, which will be changed to
	    // become the restriction we want
	    MvState currentState = createGarbageState ();
	    // fill out the restriction
	    makeRestricted (currentState, val);
	    // add the state to restrictions
	    restrictions.add (currentState);
	  }
      }

    return restrictions;
  }

  /**
   ** Creates an MvState contaning valid albeit meaningless values.
   **/
  private MvState createGarbageState ()
  {
    // get a valid lattice value
    int GARBAGE = manager.getAlgebra ().idplusIdx ();
    // create an array of restrictions of appropriate length
    int [] restrictions = new int [manager.getNumVars ()];
    for (int i = 0; i < manager.getNumVars (); i++)
      {
	restrictions [i] = ((i%2)==0) ? GARBAGE : Algebra.NO_VALUE;
      }
    //    Arrays.fill (restrictions, GARBAGE);

    return new MvState (restrictions, manager.getAlgebra ());
  }

  /** Finds a path through a decision diagram leading to 'value'
   ** recording it in 'state'
   **/
  MvState makeRestricted (MvState state, int value)
  {
    // -- nothing to do for constant nodes
    if (isConstant ()) 
      {
	Assert.assert (value == getValue ());
	return state;
      }
    
    // -- otherwise find a first child that is on the path 
    // -- to 'value' and follow it
    
    for (int i = 0; i < howManyChildren (); i++)
      {
	// -- check for a candidate
	if (getChild (i).inImage (value))
	  {
	    // -- got our candidate, set value and recurse
	    state.setValue (getVarIndex (), i);
	    return getChild (i).makeRestricted (state, value);
	  }
      }
    
    // -- hmm... this should probably be an assertion or exception
    // -- since we assume we always have what we are looking for
    return state;
    
  }
  
  /**
   ** Ensures that the state is one of the states that produce the
   ** desired value (assuming that it is possible to reach that value).
   **/
//   private void makeRestricted (MvState state, int value)
//   {
//     if (isConstant) return; // our job here is done
    
//     // find the child that has value in its image, thus deciding which
//     // value to assign to the variable represented by the current node
//     int val = 0;
//     if (getVarIndex () % 2 == 0) // kind of HACKy....
//       {
// 	while (val < numChildren)
// 	  {
// 	    if (children [val].inImage (value))
// 	      {
// 		state.setValue (getVarIndex (), val);
// 		// recurse!	   
// 		children [val].makeRestricted (state, value);
// 		break;
// 	      }
// 	    val++;
// 	  }
//       }
//     else // just keep going
//       children [val].makeRestricted (state, value);
//   }

//    /**
//     ** This function will return a restriction (not necessarily
//     ** complete) for each lattice value in which you are interested.
//     **
//     ** @param interested -- bitset indicating the lattice values in
//     ** which you are interested.
//     ** @author CDTW
//     **/
//    public Set findStates (BitSet interested)
//      {
//        log.debug ("Finding states for: "+interested);
//        Set traces = new HashSet ();
//        MvState currentTrace = 
//  	new MvState (manager.getNumVars (), manager.getAlgebra ());
//        log.debug ("Number of vars " + manager.getNumVars ());   
//        findRestrictions (interested, traces, currentTrace, 0);
//        return traces;
//      };
  

//    // XXX AG: This is very very very very ugly and should die as soon
//    // XXX AG: as possible including all of its supporting material such as
//    // XXX AG: bitsets!
//    /** helps the other findRestrictions; this is the recursive function. */
//    void findRestrictions (BitSet possible, Set traces, 
//  			 MvState currentTrace, int iteration)
//      {
//        if (isConstant ())
//  	{
//  	  log.debug ("       Constant node at iteration " + iteration);
//  	  FillOutSingleTrace (currentTrace, iteration);
//  	  traces.add (currentTrace);
//  	  return;
//  	}

//        for (int i=0; i <  numChildren; i++)
//  	{
//  	  BitSet child = (BitSet)getChild (i).getImage ().clone ();
//  	  child.and (possible);

//  	  //if (!child.equals (new BitSet (numChildren)))
//  	  if (child.length () != 0)
//  	    {
//  	      log.debug 
//  		("   Child hit at iteration "+iteration+" child "+i);
//  	      log.debug
//  		("    Bitset: "+child);

//  	      possible.xor (child);

//  	      MvState newTrace = ((MvState)currentTrace.clone ());
//  	      // XXX The below should be replaced with the further 
//  	      // XXX (commented out) below

//  	      // XXX AG removed: newTrace.setValue (iteration*2, i);
//  	      newTrace.setValue (getVarIndex (), i);

//  	      //newTrace.setValue 
//  	      // (propOrdering.getUnPrimedProps () [iteration], i);
	      
//  	      getChild (i).findRestrictions (child, traces, 
//  					     newTrace, iteration + 1);
//  	    }
//  	}
//      }

//    /**
//     * fill the rest of the state with garbage starting at iteration 
//     */
//    void FillOutSingleTrace (MvState currentTrace, int iteration)
//    {
//      int GARBAGE = 1;
//      for (int i = iteration; i < currentTrace.size ()/2; i++)
//        currentTrace.setValue (2 * i, GARBAGE);
//    }
  

  public void debugDump() 
  {
    debugDumpRecur(0);
  }

  void debugDumpRecur(int i)
  {
    for (int j=0; j<i; j++)
      System.out.print("\t");
    System.out.println(this.toString());
    visited=true;
    if (!isConstant)
      for (int j=0; j<numChildren; j++) {
	System.out.print("Child #"+j+": ");
	children[j].debugDumpRecur(i+1);
      }

  }

  public static void main (String argv[])
  {
    MDDNode m1, m2, m3, m4, m5;
    m1 = new MDDNode(2, 3);
    m2 = new MDDNode(2, 2);
    MDDNode chlist[] = {m1, m2, m2, m1};
    m3 = new MDDNode(0, chlist);
    MDDNode chlist2[] = {m2, m2, m1, m1};
    m4 = new MDDNode(0, chlist2);
    MDDNode chlist3[] = {m3, m4, m4, m3};
    m5 = new MDDNode(1, chlist3);
    m5.debugDump();
    //	m4.debugDump();
	
  }


  public void writeToDaVinci (String fileName, PropOrderIf propOrder)
  {
    try {
      FileWriter out = new FileWriter (fileName);
      out.write (toDaVinciString (propOrder));
      out.close ();
    }
    catch (Exception ex)
      {
	Assert.assert (ex);
      }
    
  }
  
    public String toDaVinciString (PropOrderIf propOrder)
    {
      DGraph tmp = PrettyGraphMDDVisitor.getPrettyRep (this, propOrder);
      
      DVGenNodeVisitor dvg = 
        new DVGenNodeVisitor (new DVGenEdgeVisitor ());
	
      for (Iterator it = DGDepthTraverse.getIterator (tmp); 
  	 it.hasNext ();)
        dvg.visit ((DGNode)it.next (), null);
	
	  
      return dvg.getSB ().toString ();
      //      return "";
    }
  
}





