package edu.toronto.cs.smv;

import java.util.*;
import edu.toronto.cs.util.*;

public interface EnumeratedTypes 
{

  public static final EnumeratedTypes EMPTY_TYPES = 
    new EmptyEnumeratedTypes ();
  
  void addVariable(String vName, String[] range);
  
  /** Getting information: */

  /** Is this a variable of enumerated type? */
  boolean isEnumeratedVariable(String varName);

  /** How many elements in the range? */
  int getRangeSize(String varName);
  /** How many bits in the encoding? */
  int getRangeBits(String varName);
  /** Get the actual range as a list. */
  List getRange(String varName);
  /** Is valName in the range of varName */
  boolean variableHasValue(String varName, String valName);
  /** Return the value that encoding represents. */
  String valueEncoded(String varName, int encoding);
  /** Return (an!) encoding for valName. Pick one arbitrarily, just use
      the same one everytime, if there's more than one.*/
  int getEncoding(String varName, String valName); 
  /** Useful for comparisons. */
  int[] getAllEncodings(String varName, String valName);
  
  
}

class EmptyEnumeratedTypes implements EnumeratedTypes
{
  public EmptyEnumeratedTypes ()
  {
  }
  
  public void addVariable(String vName, String[] range)
  {
    Assert.assert ("addVariable is not implemented in " + 
		   EmptyEnumeratedTypes.class.getName ());
  }
  
  
  public boolean isEnumeratedVariable(String varName)
  {
    return false;
  }


  public int getRangeSize(String varName)
  {
    return 2;
  }
  
  public int getRangeBits(String varName)
  {
    return 1;
  }
  
  public List getRange(String varName)
  {
    return Collections.EMPTY_LIST;
  }
   
  public boolean variableHasValue(String varName, String valName)
  {
    return false;
  }
  
  /** Return the value that encoding represents. */
  public String valueEncoded(String varName, int encoding)
  {
    return null;
  }
  
  public int getEncoding(String varName, String valName)
  {
    Assert.assert ("getEncoding is not implemented in " + 
		   EmptyEnumeratedTypes.class.getName ());
    return 0;
  }
  
  /** Useful for comparisons. */
  public int[] getAllEncodings(String varName, String valName)
  {
    Assert.assert ("getAllEncodings is not implemented in " + 
		   EmptyEnumeratedTypes.class.getName ());
    return null;
  }
}
