package edu.toronto.cs.smv;

import java.util.*;
import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.algebra.*;


/*** Class representing an SMV module that contains submodules, 
 *** like an elevator and its buttons
 ****/
public abstract class SMVCombinedModule extends SMVModule
{

  protected List subModules;

  public SMVCombinedModule (String _name)
  {
    super (_name);
    subModules = new ArrayList ();
  }
  

  public void addSubModules (Collection modules)
  {
    subModules.addAll (modules);
  }
  
  public void addSubModule (SMVModule module)
  {
    subModules.add (module);
  }
  

  public List getSpecs ()
  {
    List specs = new ArrayList ();
    for (Iterator it = subModules.iterator (); it.hasNext ();)
      specs.addAll (((SMVModule)it.next ()).getSpecs ());
    return specs;
  }
  

  public void setMvSetFactory (MvSetFactory _ddManager)
  {
    super.setMvSetFactory (_ddManager);
    for (Iterator it = subModules.iterator (); it.hasNext ();)
      ((SMVModule)it.next ()).setMvSetFactory (_ddManager);
  }
  
  public void setLattice (FiniteLattice _lattice)
  {
    super.setLattice (_lattice);
    
    for (Iterator it = subModules.iterator (); it.hasNext ();)
      ((SMVModule)it.next ()).setLattice (_lattice);
  }
  
    
  public String[] getVars ()
  {
    List allVars = new ArrayList ();
    allVars.addAll (Arrays.asList (super.getVars ()));
    
    for (Iterator it = subModules.iterator (); it.hasNext ();)
      allVars.addAll (Arrays.asList (((SMVModule)it.next ()).getVars ()));
	
    return (String[])allVars.toArray (new String [allVars.size ()]);
  }
  


  // -- set variable ordering
  public void setVarOrdering (PropOrderIf ordering)
  {
    super.setVarOrdering (ordering);
    for (Iterator it = subModules.iterator (); it.hasNext ();)
      ((SMVModule)it.next ()).setVarOrdering (ordering);
  }
  
  
  // -- computes an MvSet representation of the transition relation
  public MvSet computeTrans ()
  {
    MvSet result = ddManager.createConstant (top);
    for (Iterator it = subModules.iterator (); it.hasNext ();)
      result = result.and (((SMVModule)it.next ()).computeTrans ());
    return result;
    
  }
  
  public MvState initializeInitState (MvState _initState)
  {
    MvState initState = _initState;
    for (Iterator it = subModules.iterator (); it.hasNext ();)
      ((SMVModule)it.next ()).initializeInitState (initState);
    return initState;
  }
  

}



