package edu.toronto.cs.xkripke;

import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.xkripke.XKripke.*;
import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mdd.*;
import edu.toronto.cs.algebra.*;

import java.util.Iterator;

// -- given a XKripke model creates an MDD for it
public class XKripkeMDDBuilder
{
  /*****
   ***** Usage
   *****
   *****  1) Create MDDManager 
   *****  2) Create XKrpike (use XKripkeFactory for example)
   *****  3) Create propOrder 
   *****      for example: new AlternatePropOrder (kripke.getPropNames ())
   *****  4) call buildMDD and something will happen :)
   */

  /***
   *** mddMgr -- needed to create MDD nodes
   *** kripke -- the XKripke model to convert
   *** propOrder -- interface to order the atomic props
   ***/
  public static MDDNode buildMDD (MDDManager mddMgr, XKripke kripke, 
				  PropOrderIf propOrder)
  {
    // -- we build each transition individually and then 
    // -- join all of them together

    // -- start with the identity on JOIN
    MDDNode result = mddMgr.getZero ();
    for (Iterator it = kripke.getTransitions ().iterator (); it.hasNext ();)
      result = mddMgr.Apply (result, 
			     buildTransition (mddMgr, kripke.getLattice (),
					      (XKripkeTransition)it.next (), 
					      propOrder), 
			     FiniteLattice.JOIN);
    return result;
  }

  public static MDDNode buildTransition (MDDManager mddMgr, 
					 FiniteLattice lattice,
					 XKripkeTransition trans,
					 PropOrderIf propOrder)
  {
    // -- identity for MEET
    MDDNode result = mddMgr.getOne ();
    
    int idx = 0; // -- variable indexing

    for (Iterator it = propOrder.orderTransition (trans); it.hasNext ();)
      {
	XKripkeProp prop = (XKripkeProp)it.next ();
	int latticeValue = lattice.getElementIdxByKey (prop.getValue ());
	result = mddMgr.Apply (result, 
			       mddMgr.getVarEquals (idx++, latticeValue),
			       FiniteLattice.MEET);
      }
    int transitionValue = lattice.getElementIdxByKey (trans.getValue ());
    result = mddMgr.setEqualTo (result, transitionValue);
    return result;
  }


  public static void main (String[] args) throws Exception 
  {
    XKripke kripke = XKripkeFactory.parse (args [0]);
    PropOrderIf propOrdering = 
      new AlternatePropOrder (kripke.getPropNames ());

    

    MDDManager mddManager = new MDDMgr (propOrdering.size (), 
					kripke.getLattice ());
    MDDNode mdd = buildMDD (mddManager, kripke, propOrdering);
    System.out.println ("Is final mdd constant: " + mdd.isConstant ());
    System.out.println ("Writing mdd into systemMdd.daVinci");
    MDDNodeToDaVinci.mddToDaVinci (mdd, propOrdering, "systemMdd.daVinci");
    System.out.println ("Property ordering is as follows:");
    
    int size = propOrdering.size ();
    for (int i = 0; i < size; i++)
      {
	System.out.print ("#" + i + " = " + propOrdering.getPropName (i));
	if (propOrdering.isPostImage (i)) 
	  System.out.print ("'");
	System.out.println ();	
      }
    
    
  }
  
  
}



